@@ -25,28 +25,63 @@
 #include <stdio.h>
 
 #include <sys/wait.h>
+#include <signal.h>
 #include <unistd.h>
+#include <math.h>
+
+#include <libart_lgpl/libart.h>
 
 #include "bug-buddy.h"
 #include "util.h"
-#include "glade-druid.h"
+
+#define d(x)
+
+static gint
+animate (gpointer data)
+{
+	double affine[6];
+
+	art_affine_rotate (affine, -36);
+	gnome_canvas_item_affine_relative (druid_data.throbber, affine);
+	return TRUE;
+}
+
+static void
+start_animation (void)
+{
+	g_return_if_fail (druid_data.throbber_id == 0);
+
+	druid_data.throbber_id = gtk_timeout_add (150, animate, NULL);
+}
+
+static void
+stop_animation (void)
+{
+	g_return_if_fail (druid_data.throbber_id != 0);
+
+	gtk_timeout_remove (druid_data.throbber_id);
+	druid_data.throbber_id = 0;
+}
 
 void
-start_gdb ()
+start_gdb (void)
 {
 	static gchar *old_app = NULL;
 	static gchar *old_extra = NULL;
 	static CrashType old_type = -1;
-	
 	gchar *app = NULL, *extra = NULL;
 
-	g_message (_("obtaining stack trace... (%d)"), druid_data.crash_type);
+	d(g_message (_("Obtaining stack trace... (%d)"), druid_data.crash_type));
+
+	if (druid_data.crash_type == CRASH_NONE)
+		return;
 	
 	switch (druid_data.crash_type) {
 	case CRASH_DIALOG:
-		app = gtk_entry_get_text (GTK_ENTRY (APP_FILE));
-		extra = gtk_entry_get_text (GTK_ENTRY (CRASHED_PID));
+		app = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-binary-entry")));
+		extra = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-pid-entry")));
 		druid_data.app_pid = atoi (extra);
+		kill (druid_data.app_pid, SIGCONT);
 		if (druid_data.explicit_dirty ||
 		    (old_type != CRASH_DIALOG) ||
 		    (!old_app || strcmp (app, old_app)) ||
@@ -55,7 +90,7 @@ start_gdb ()
 		}
 		break;
 	case CRASH_CORE:
-		extra = gtk_entry_get_text (GTK_ENTRY (CORE_FILE));
+		extra = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-core-entry")));
 		if (druid_data.explicit_dirty ||
 		    (old_type != CRASH_CORE) ||
 		    (!old_extra || strcmp (extra, old_extra))) {
@@ -77,10 +112,10 @@ start_gdb ()
 }
 
 void
-stop_gdb ()
+stop_gdb (void)
 {
 	if (!druid_data.ioc) {
-		g_message (_("gdb has already exited"));
+		d(g_message (_("gdb has already exited")));
 		return;
 	}
 	
@@ -88,14 +123,17 @@ stop_gdb ()
 	waitpid (druid_data.gdb_pid, NULL, 0);
 	
 	druid_data.gdb_pid = 0;
-	gnome_druid_set_buttons_sensitive (GNOME_DRUID (druid_data.the_druid),
-					   TRUE, TRUE, TRUE);
-	gnome_animator_stop (GNOME_ANIMATOR (GDB_ANIM));
+
+	druid_set_sensitive (TRUE, TRUE, TRUE);
+	stop_animation ();
+
 	druid_data.fd = 0;
 	druid_data.ioc = NULL;
-	gtk_widget_set_sensitive (GTK_WIDGET (STOP_BUTTON), FALSE);
-	gtk_widget_set_sensitive (GTK_WIDGET (REFRESH_BUTTON),
-				  TRUE);
+	gtk_widget_set_sensitive (GTK_WIDGET (GET_WIDGET ("gdb-stop")), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (GET_WIDGET ("gdb-go")), TRUE);
+
+	if (GTK_TOGGLE_BUTTON (GET_WIDGET ("gdb-continue-toggle"))->active)
+		druid_set_state (STATE_DESC);
 	return;
 }
 
@@ -148,6 +186,11 @@ get_trace_from_core (const gchar *core_file)
 		return;
 	}	
 
+	if (!popt_data.app_file) {
+		d(g_message ("Setting binary: %s", binary));
+		popt_data.app_file = g_strdup (binary);
+	}
+
 	get_trace_from_pair (binary, core_file);
 	g_free (binary);
 }
@@ -155,7 +198,7 @@ get_trace_from_core (const gchar *core_file)
 static gboolean
 handle_gdb_input (GIOChannel *ioc, GIOCondition condition, gpointer data)
 {	
-	GtkWidget *w;
+	GtkWidget *w = NULL;
 	gchar buf[1024];
 	guint len;
 	
@@ -171,12 +214,12 @@ handle_gdb_input (GIOChannel *ioc, GIOCondition condition, gpointer data)
 	case G_IO_ERROR_AGAIN:
 		goto gdb_try_read;
 	default:
-		g_warning (_("Error on read... aborting"));
+		d(g_warning (_("Error on read... aborting")));
 		stop_gdb ();
 		return FALSE;
 	}
 
-	w = GDB_TEXT;
+	w = GET_WIDGET ("gdb-text");
 	gtk_text_set_point (GTK_TEXT (w),
 			    gtk_text_get_length (GTK_TEXT (w)));
 	gtk_text_insert (GTK_TEXT (w), 
@@ -190,12 +233,23 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 {
 	GtkWidget *d;
 	int fd;
-	const char *args[] = { "gdb", 
-			       "--batch", 
-			       "--quiet",
-			       "--command=" BUDDY_DATADIR "/gdb-cmd",
-			       NULL, NULL, NULL };
-	args[5] = extra;
+	char *app2;
+	char *args[] = { "gdb", 
+			 "--batch", 
+			 "--quiet",
+			 "--command=" BUDDY_DATADIR "/gdb-cmd",
+			 NULL, NULL, NULL };
+	args[0] = gnome_is_program_in_path ("gdb");
+	args[5] = (char *)extra;
+
+	if (!args[0]) {
+		d(g_message ("Path: %s", getenv ("PATH")));
+		gnome_dialog_run_and_close (
+			GNOME_DIALOG (
+				gnome_error_dialog (_("GDB could not be found on your system.\n"
+						      "Debugging information will not be obtained."))));
+		return;
+	}
 
 	if (!app || !extra || !*app || !*extra)
 		return;
@@ -203,30 +257,32 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 	/* FIXME: we should probably be fully expanding the link to
 	   see if it is a directory, but unix sucks and i am lazy */
 	if (g_file_test (app, G_FILE_TEST_ISFILE | G_FILE_TEST_ISLINK))
-		app = g_strdup (app);
+		app2 = g_strdup (app);
 	else
-		app = gnome_is_program_in_path (app);
+		app2 = gnome_is_program_in_path (app);
 
-	if (!app)
-		return;
-
-	args[4] = app;
-
-	g_message ("About to debug '%s'", app);
-	
-	if (!g_file_exists (BUDDY_DATADIR "/gdb-cmd")) {
-		d = gnome_error_dialog (_("Could not find the gdb-cmd file.\n"
-					  "Please try reinstalling bug-buddy."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
-		g_free (app);
+	if (!app2) {
+		g_free (args[0]);
 		return;
 	}
 
-	druid_data.gdb_pid = start_commandv (args, &fd);
+	args[4] = app2;
+
+	d(g_message ("About to debug '%s'", app2));
+	
+	if (!g_file_exists (BUDDY_DATADIR "/gdb-cmd")) {
+		d = gnome_error_dialog (_("Could not find the gdb-cmd file.\n"
+					  "Please try reinstalling Bug Buddy."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		g_free (app2);
+		return;
+	}
+
+	druid_data.gdb_pid = start_commandv ((const char **)args, &fd);
 	if (druid_data.gdb_pid == -1) {
 		d = gnome_error_dialog (_("Error on fork()."));
 		gnome_dialog_run_and_close (GNOME_DIALOG (d));
-		g_free (app);
+		g_free (app2);
 		return;
 	}
 	
@@ -235,17 +291,17 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 	g_io_add_watch (druid_data.ioc, G_IO_IN | G_IO_HUP, 
 			handle_gdb_input, NULL);
 	g_io_channel_unref (druid_data.ioc);
-	gtk_editable_delete_text (GTK_EDITABLE (GDB_TEXT), 0, -1);
-	gnome_druid_set_buttons_sensitive (GNOME_DRUID (THE_DRUID),
-					   FALSE, FALSE, TRUE);
-	gnome_animator_start (GNOME_ANIMATOR (GDB_ANIM));
-	gtk_widget_set_sensitive (GTK_WIDGET (STOP_BUTTON), TRUE);
-	gtk_widget_set_sensitive (GTK_WIDGET (REFRESH_BUTTON),
-				  FALSE);
+	gtk_editable_delete_text (GTK_EDITABLE (GET_WIDGET ("gdb-text")), 0, -1);
+
+	druid_set_sensitive (FALSE, FALSE, TRUE);
+	start_animation ();
+
+	gtk_widget_set_sensitive (GTK_WIDGET (GET_WIDGET ("gdb-stop")), TRUE);
+	gtk_widget_set_sensitive (GTK_WIDGET (GET_WIDGET ("gdb-go")), FALSE);
 
 	druid_data.explicit_dirty = FALSE;
 
-	g_free (app);
+	g_free (app2);
 }
 
 
